@@ -1,6 +1,6 @@
 package Hubot::Adapter::Irc;
 {
-  $Hubot::Adapter::Irc::VERSION = '0.1.10';
+  $Hubot::Adapter::Irc::VERSION = '0.2.0';
 }
 use Moose;
 use namespace::autoclean;
@@ -194,7 +194,7 @@ sub run {
             my ( $channel, $mode, $target ) = @{ $ircmsg->{params} };
 
             $self->robot->mode( $mode || '' )
-                if $target eq $self->robot->name;
+                if $target && $target eq $self->robot->name;
         },
     );
 
