@@ -1,6 +1,6 @@
 package Hubot::Robot;
 {
-  $Hubot::Robot::VERSION = '0.1.10';
+  $Hubot::Robot::VERSION = '0.2.0';
 }
 
 use Moose;
@@ -8,6 +8,8 @@ use namespace::autoclean;
 
 use Pod::Usage;
 
+use AnyEvent;
+use AnyEvent::HTTPD;
 use AnyEvent::HTTP::ScopedClient;
 
 use Hubot::User;
@@ -15,19 +17,19 @@ use Hubot::Brain;
 use Hubot::Listener;
 use Hubot::TextListener;
 
-has 'name' => ( is => 'rw', isa => 'Str' );
+has 'name'  => ( is => 'rw', isa => 'Str' );
 has 'alias' => ( is => 'rw', isa => 'Str' );
-has 'mode' => ( is => 'rw', isa => 'Str', default => '' );
+has 'mode'  => ( is => 'rw', isa => 'Str', default => '' );
 has 'adapter' => ( is => 'rw' );
 has 'brain' => (
-    is => 'ro',
-    isa => 'Hubot::Brain',
+    is      => 'ro',
+    isa     => 'Hubot::Brain',
     default => sub { Hubot::Brain->new }
 );
 has '_helps' => (
     traits  => ['Array'],
-    is => 'rw',
-    isa => 'ArrayRef[Str]',
+    is      => 'rw',
+    isa     => 'ArrayRef[Str]',
     default => sub { [] },
     handles => {
         helps   => 'elements',
@@ -36,8 +38,8 @@ has '_helps' => (
 );
 has '_commands' => (
     traits  => ['Array'],
-    is => 'rw',
-    isa => 'ArrayRef[Str]',
+    is      => 'rw',
+    isa     => 'ArrayRef[Str]',
     default => sub { [] },
     handles => {
         commands   => 'elements',
@@ -46,8 +48,8 @@ has '_commands' => (
 );
 has '_listeners' => (
     traits  => ['Array'],
-    is => 'rw',
-    isa => 'ArrayRef[Hubot::Listener]',
+    is      => 'rw',
+    isa     => 'ArrayRef[Hubot::Listener]',
     default => sub { [] },
     handles => {
         listeners   => 'elements',
@@ -55,36 +57,71 @@ has '_listeners' => (
     }
 );
 
+## Ping Watcher
+has 'pw'    => ( is => 'rw' );
+has 'httpd' => ( is => 'rw' );
+
 sub BUILD {
     my $self = shift;
-    $self->loadAdapter($self->adapter)
+
+    $self->setupHerokuPing;
+    $self->loadAdapter( $self->adapter );
+}
+
+sub setupHerokuPing {
+    my $self = shift;
+
+    my $httpd = AnyEvent::HTTPD->new( port => $ENV{PORT} || 8080 );
+    $httpd->reg_cb(
+        '/hubot/ping' => sub {
+            my ( $httpd, $req ) = @_;
+            $req->respond( { content => [ 'text/plain', "pong" ] } );
+        }
+    );
+
+    $self->httpd($httpd);
+
+    my $herokuUrl = $ENV{HEROKU_URL} || return;
+    $herokuUrl =~ s{/?$}{/hubot/ping};
+
+    $self->pw(
+        AE::timer 0,
+        1200000,
+        sub {
+            AnyEvent::HTTP::ScopedClient->new($herokuUrl)
+                ->post( sub { print "Keep alive ping!\n" if $ENV{DEBUG} } );
+        }
+    );
 }
 
 sub loadAdapter {
-    my ($self, $adapter) = @_;
-    ## TODO: HUBOT_DEFAULT_ADAPTERS
+    my ( $self, $adapter ) = @_;
+
     $adapter = "Hubot::Adapter::" . ucfirst($adapter);
     eval "require $adapter; 1";
     if ($@) {
         print STDERR "Cannot load adapter $adapter - $@\n";
-    } else {
-        $self->adapter($adapter->new({ robot => $self }));
+    }
+    else {
+        $self->adapter( $adapter->new( { robot => $self } ) );
     }
 }
 
 sub run { shift->adapter->run }
 
 sub userForId {
-    my ($self, $id, $options) = @_;
+    my ( $self, $id, $options ) = @_;
     my $user = $self->brain->{data}{users}{$id};
     unless ($user) {
-        $user = Hubot::User->new({ id => $id, %$options });
+        $user = Hubot::User->new( { id => $id, %$options } );
         $self->brain->{data}{users}{$id} = $user;
     }
 
     my $options_room = $options->{room} || '';
-    if ($options_room && (!$user->{room} || $user->{room} ne $options_room)) {
-        $user = Hubot::User->new({ id => $id, %$options });
+    if ( $options_room
+        && ( !$user->{room} || $user->{room} ne $options_room ) )
+    {
+        $user = Hubot::User->new( { id => $id, %$options } );
         $self->brain->{data}{users}{$id} = $user;
     }
 
@@ -92,11 +129,11 @@ sub userForId {
 }
 
 sub userForName {
-    my ($self, $name) = @_;
+    my ( $self, $name ) = @_;
     my $result;
-    for my $k (keys %{ $self->brain->{data}{users} }) {
+    for my $k ( keys %{ $self->brain->{data}{users} } ) {
         my $userName = $self->brain->{data}{users}{$k}{name};
-        if (lc $userName eq lc $name) {
+        if ( lc $userName eq lc $name ) {
             $result = $self->brain->{data}{users}{$k};
         }
     }
@@ -105,11 +142,12 @@ sub userForName {
 }
 
 sub usersForFuzzyRawName {
-    my ($self, $fuzzyName) = @_;
+    my ( $self, $fuzzyName ) = @_;
     my $lowerFuzzyName = lc $fuzzyName;
     my @users;
-    while (my ($key, $user) = each %{ $self->brain->{data}{users} || {} }) {
-        if (lc($user->{name}) =~ m/^$lowerFuzzyName/) {
+    while ( my ( $key, $user ) = each %{ $self->brain->{data}{users} || {} } )
+    {
+        if ( lc( $user->{name} ) =~ m/^$lowerFuzzyName/ ) {
             push @users, $user;
         }
     }
@@ -118,11 +156,11 @@ sub usersForFuzzyRawName {
 }
 
 sub usersForFuzzyName {
-    my ($self, $fuzzyName) = @_;
-    my @matchedUsers = $self->usersForFuzzyRawName($fuzzyName);
+    my ( $self, $fuzzyName ) = @_;
+    my @matchedUsers   = $self->usersForFuzzyRawName($fuzzyName);
     my $lowerFuzzyName = lc $fuzzyName;
     for my $user (@matchedUsers) {
-        return $user if lc($user->{name}) eq $lowerFuzzyName;
+        return $user if lc( $user->{name} ) eq $lowerFuzzyName;
     }
 
     return @matchedUsers;
@@ -135,7 +173,7 @@ sub shutdown {
 }
 
 sub loadHubotScripts {
-    my ($self, $scripts) = @_;
+    my ( $self, $scripts ) = @_;
     ## TODO: Debug Message
     # print "Loading hubot-scripts\n" if $ENV{DEBUG};
     for my $script (@$scripts) {
@@ -144,68 +182,73 @@ sub loadHubotScripts {
 }
 
 sub loadFile {
-    my ($self, $script) = @_;
+    my ( $self, $script ) = @_;
     my $full = "Hubot::Scripts::$script";
     eval "require $full; 1";
     $full->load($self);
     if ($@) {
         print STDERR "Unable to load $full: $@\n";
-    } else {
+    }
+    else {
         $self->parseHelp($full);
     }
 }
 
 sub parseHelp {
-    my ($self, $module) = @_;
+    my ( $self, $module ) = @_;
     $module =~ s{::}{/}g;
-    my $fullpath = $INC{$module . '.pm'};
+    my $fullpath = $INC{ $module . '.pm' };
 
     open my $fh, '>', \my $usage or die "Couldn't open filehandle: $!\n";
-    pod2usage({
-        -input   => $fullpath,
-        -output  => $fh,
-        -exitval => 'noexit',
-    });
+    pod2usage(
+        {   -input   => $fullpath,
+            -output  => $fh,
+            -exitval => 'noexit',
+        }
+    );
 
     $usage =~ s/^Usage://;
     $usage =~ s/(^\s+|\s+$)//gm;
-    $self->addHelp($_) for split(/\n/, $usage);
+    $self->addHelp($_) for split( /\n/, $usage );
 
     $module =~ s{Hubot/Scripts/}{};
     $self->addCommand($module);
 }
 
 sub hear {
-    my ($self, $regex, $callback) = @_;
-    $self->addListener(new Hubot::TextListener(
-        robot => $self,
-        regex => $regex,
-        callback => $callback
-    ));
+    my ( $self, $regex, $callback ) = @_;
+    $self->addListener(
+        new Hubot::TextListener(
+            robot    => $self,
+            regex    => $regex,
+            callback => $callback
+        )
+    );
 }
 
 sub respond {
-    my ($self, $regex, $callback) = @_;
+    my ( $self, $regex, $callback ) = @_;
 
     my $index = index "$regex", ':';
-    my $stringRegex = substr "$regex", ($index + 1), -1;
+    my $stringRegex = substr "$regex", ( $index + 1 ), -1;
     my $first = substr $stringRegex, 0, 1;
 
     ## TODO: $^ 에 따른 분기; perl version 에 따라서 Regex object 의 modifier 위치가 달라짐
-    my $modifiers = '';
+    my $modifiers    = '';
     my $modifiersLen = $index - 3;
-    if ($modifiersLen > 0 && length $stringRegex > 3) {
-        $modifiers = substr $stringRegex, 3, $modifiersLen
+    if ( $modifiersLen > 0 && length $stringRegex > 3 ) {
+        $modifiers = substr $stringRegex, 3, $modifiersLen;
     }
 
-    if ($first eq '^') {
-        print STDERR "Anchors don't work well with respond, perhaps you want to use 'hear'\n";
+    if ( $first eq '^' ) {
+        print STDERR
+            "Anchors don't work well with respond, perhaps you want to use 'hear'\n";
         print STDERR "The regex in question was $stringRegex\n";
     }
 
     my $newRegex;
     my $name = $self->name;
-    if ($self->alias) {
+    if ( $self->alias ) {
         my $alias = $self->alias;
         $alias =~ s/[-[\]{}()\*+?.,\\^$|#\s]/\\$&/g; # escape alias for regexp
 
@@ -214,78 +257,94 @@ sub respond {
         ## 일단 modifiers 가 있다면 `i` 라고 가정하고 들어감 WTH..
         if ($modifiers) {
             $newRegex = qr/^(?:$alias[:,]?|$name[:,]?)\s*(?:$stringRegex)/i;
-        } else {
-            $newRegex = qr/^(?:$alias[:,]?|$name[:,]?)\s*(?:$stringRegex)/
         }
-    } else {
+        else {
+            $newRegex = qr/^(?:$alias[:,]?|$name[:,]?)\s*(?:$stringRegex)/;
+        }
+    }
+    else {
         if ($modifiers) {
             $newRegex = qr/^(?:$name[:,]?)\s*(?:$stringRegex)/i;
-        } else {
-            $newRegex = qr/^(?:$name[:,]?)\s*(?:$stringRegex)/
+        }
+        else {
+            $newRegex = qr/^(?:$name[:,]?)\s*(?:$stringRegex)/;
         }
     }
 
     print "$newRegex\n" if $ENV{DEBUG};
-    $self->addListener(new Hubot::TextListener(
-        robot => $self,
-        regex => $newRegex,
-        callback => $callback
-    ));
+    $self->addListener(
+        new Hubot::TextListener(
+            robot    => $self,
+            regex    => $newRegex,
+            callback => $callback
+        )
+    );
 }
 
 sub enter {
-    my ($self, $callback) = @_;
-    $self->addListener(Hubot::Listener->new(
-        robot => $self,
-        matcher => sub { ref(shift) eq 'Hubot::EnterMessage' ? 1 : () },
-        callback => $callback
-    ));
+    my ( $self, $callback ) = @_;
+    $self->addListener(
+        Hubot::Listener->new(
+            robot    => $self,
+            matcher  => sub { ref(shift) eq 'Hubot::EnterMessage' ? 1 : () },
+            callback => $callback
+        )
+    );
 }
 
 sub leave {
-    my ($self, $callback) = @_;
-    $self->addListener(Hubot::Listener->new(
-        robot => $self,
-        matcher => sub { ref(shift) eq 'Hubot::LeaveMessage' ? 1 : () },
-        callback => $callback
-    ));
+    my ( $self, $callback ) = @_;
+    $self->addListener(
+        Hubot::Listener->new(
+            robot    => $self,
+            matcher  => sub { ref(shift) eq 'Hubot::LeaveMessage' ? 1 : () },
+            callback => $callback
+        )
+    );
 }
 
 sub whisper {
-    my ($self, $callback) = @_;
-    $self->addListener(Hubot::Listener->new(
-        robot => $self,
-        matcher => sub { ref(shift) eq 'Hubot::WhisperMessage' ? 1 : () },
-        callback => $callback
-    ));
+    my ( $self, $callback ) = @_;
+    $self->addListener(
+        Hubot::Listener->new(
+            robot   => $self,
+            matcher => sub { ref(shift) eq 'Hubot::WhisperMessage' ? 1 : () },
+            callback => $callback
+        )
+    );
 }
 
 sub notice {
-    my ($self, $callback) = @_;
-    $self->addListener(Hubot::Listener->new(
-        robot => $self,
-        matcher => sub { ref(shift) eq 'Hubot::NoticeMessage' ? 1 : () },
-        callback => $callback
-    ));
+    my ( $self, $callback ) = @_;
+    $self->addListener(
+        Hubot::Listener->new(
+            robot    => $self,
+            matcher  => sub { ref(shift) eq 'Hubot::NoticeMessage' ? 1 : () },
+            callback => $callback
+        )
+    );
 }
 
 sub catchAll {
-    my ($self, $callback) = @_;
-    $self->addListener(Hubot::Listener->new(
-        robot => $self,
-        matcher => sub { ref(shift) eq 'Hubot::CatchAllMessage' ? 1 : () },
-        callback => sub {
-            my $msg = shift;
-            $msg->message($msg->message->message);
-            $callback->($msg);
-        }
-    ));
+    my ( $self, $callback ) = @_;
+    $self->addListener(
+        Hubot::Listener->new(
+            robot => $self,
+            matcher =>
+                sub { ref(shift) eq 'Hubot::CatchAllMessage' ? 1 : () },
+            callback => sub {
+                my $msg = shift;
+                $msg->message( $msg->message->message );
+                $callback->($msg);
+            }
+        )
+    );
 }
 
 sub receive {
-    my ($self, $message) = @_;
+    my ( $self, $message ) = @_;
     my $results = [];
-    for my $listener ($self->listeners) {
+    for my $listener ( $self->listeners ) {
         eval $listener->call($message);
         last if $message->done;
         if ($@) {
@@ -294,12 +353,11 @@ sub receive {
         }
     }
 
-    $self->receive(new Hubot::CatchAllMessage(
-        message => $message
-    )) if ref($message) ne 'Hubot::CatchAllMessage';
+    $self->receive( new Hubot::CatchAllMessage( message => $message ) )
+        if ref($message) ne 'Hubot::CatchAllMessage';
 }
 
-sub http { AnyEvent::HTTP::ScopedClient->new($_[1]) }
+sub http { AnyEvent::HTTP::ScopedClient->new( $_[1] ) }
 
 __PACKAGE__->meta->make_immutable;
 
